from datetime import datetime

from cpl_core.database import TableABC

from bot_data.model.user import User


class UserJoinedGameServer(TableABC):
    def __init__(
        self,
        user: User,
        game_server: str,
        joined_on: datetime,
        leaved_on: datetime = None,
        created_at: datetime = None,
        modified_at: datetime = None,
        id=0,
    ):
        self._id = id
        self._user = user
        self._game_server = game_server
        self._joined_on = joined_on
        self._leaved_on = leaved_on

        TableABC.__init__(self)
        self._created_at = created_at if created_at is not None else self._created_at
        self._modified_at = modified_at if modified_at is not None else self._modified_at

    @property
    def id(self) -> int:
        return self._id

    @property
    def user(self) -> User:
        return self._user

    @property
    def game_server(self) -> str:
        return self._game_server

    @property
    def time(self) -> float:
        return round((self.leaved_on - self.joined_on).total_seconds() / 3600, 2)

    @property
    def joined_on(self) -> datetime:
        return self._joined_on

    @joined_on.setter
    def joined_on(self, value: datetime):
        self._modified_at = datetime.now()
        self.joined_on = value

    @property
    def leaved_on(self) -> datetime:
        return self._leaved_on

    @leaved_on.setter
    def leaved_on(self, value: datetime):
        self._modified_at = datetime.now()
        self._leaved_on = value

    @staticmethod
    def get_select_all_string() -> str:
        return str(
            f"""
            SELECT * FROM `UserJoinedGameServer`;
        """
        )

    @staticmethod
    def get_select_by_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `UserJoinedGameServer`
            WHERE `Id` = {id};
        """
        )

    @staticmethod
    def get_select_by_user_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `UserJoinedGameServer`
            WHERE `UserId` = {id};
        """
        )

    @staticmethod
    def get_select_active_by_user_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `UserJoinedGameServer`
            WHERE `UserId` = {id}
            AND `LeavedOn` IS NULL;
        """
        )

    @property
    def insert_string(self) -> str:
        if self._leaved_on is not None:
            return str(
                f"""
                INSERT INTO `UserJoinedGameServer` (
                    `UserId`, `GameServer`, `JoinedOn`, `LeavedOn`, `CreatedAt`, `LastModifiedAt`
                ) VALUES (
                    {self._user.id},
                    '{self._game_server}',
                    '{self._joined_on}',
                    '{self._leaved_on}',
                    '{self._created_at}',
                    '{self._modified_at}'
                );
            """
            )
        else:
            return str(
                f"""
                INSERT INTO `UserJoinedGameServer` (
                    `UserId`, `GameServer`, `JoinedOn`, `CreatedAt`, `LastModifiedAt`
                ) VALUES (
                    {self._user.id},
                    '{self._game_server}',
                    '{self._joined_on}',
                    '{self._created_at}',
                    '{self._modified_at}'
                );
            """
            )

    @property
    def udpate_string(self) -> str:
        return str(
            f"""
            UPDATE `UserJoinedGameServer`
            SET `LeavedOn` = '{self._leaved_on}',
            `LastModifiedAt` = '{self._modified_at}'
            WHERE `Id` = {self._id};
        """
        )

    @property
    def delete_string(self) -> str:
        return str(
            f"""
            DELETE FROM `UserJoinedGameServer`
            WHERE `Id` = {self._id};
        """
        )

    @staticmethod
    def delete_by_user_id_string(id: int) -> str:
        return str(
            f"""
            DELETE FROM `UserJoinedGameServer`
            WHERE `UserId` = {id}
    """
        )
