export class Queries {

  static guildsQuery = `
    query GuildsQuery($id: ID) {
      guilds(filter: {id: $id}) {
        id
        name

        channels {
          id
          name
          type
        }
        roles {
          id
          name
        }
        emojis {
          id
          name
          url
        }
      }
    }
  `;

  static serversQuery = `
    query ServerInfo($filter: ServerFilter, $page: Page, $sort: Sort) {
      servers(filter: $filter, page: $page, sort: $sort) {
        id
        name
        iconURL
        userCount
        clients{
          id
          discordId
          name
          sentMessageCount
          receivedMessageCount
          deletedMessageCount
          receivedCommandCount
          movedUsersCount
        }
      }
    }
  `;

  static levelQuery = `
    query LevelsList($filter: LevelFilter, $page: Page, $sort: Sort) {
      levelCount
      levels(filter: $filter, page: $page, sort: $sort) {
        id
        name
        color
        minXp
        permissions
        server {
          id
          name
        }
        createdAt
        modifiedAt
      }
    }
  `;

  static usersQuery = `
    query UsersList($filter: UserFilter, $page: Page, $sort: Sort) {
      userCount
      users(filter: $filter, page: $page, sort: $sort) {
        id
        discordId
        name
        xp
        ontime
        level {
          id
          name
        }
        server {
          id
          name
        }
        leftServer

        joinedServerCount
        joinedServers {
          id
        }

        joinedVoiceChannelCount
        joinedVoiceChannels {
          id
          channelId
          channelName
        }

        userJoinedGameServerCount
        userJoinedGameServers {
          id
          gameServer
        }

        createdAt
        modifiedAt
      }
    }
  `;

  static singleUserQuery = `
    query singleUserQuery($filter: UserFilter) {
      users(filter: $filter) {
        id
        discordId
        name
        xp
        ontime
        minecraftId
        level {
          id
          name
        }
        leftServer
        server {
          id
          name
        }

        joinedServerCount
        joinedServers {
          id
          joinedOn
          leavedOn
        }

        joinedVoiceChannelCount
        joinedVoiceChannels {
          id
          channelId
          channelName
          time
          joinedOn
          leavedOn
        }

        userJoinedGameServerCount
        userJoinedGameServers {
          id
          gameServer
          time
          joinedOn
          leavedOn
        }

        createdAt
        modifiedAt
      }
    }
  `;

  static autoRolesQuery = `
    query AutoRoleQuery($serverId: ID, $filter: AutoRoleFilter, $page: Page, $sort: Sort) {
      servers(filter: {id: $serverId}) {
        autoRoleCount
        autoRoles(filter: $filter, page: $page, sort: $sort) {
          id
          channelId
          channelName
          messageId
          autoRoleRuleCount

          server {
            id
          }

          createdAt
          modifiedAt
        }
      }
    }
  `;

  static autoRoleRulesQuery = `
    query AutoRoleRuleQuery($serverId: ID, $autoRoleId: ID, $filter: AutoRoleRuleFilter, $page: Page, $sort: Sort) {
      servers(filter: {id: $serverId}) {
        autoRoles(filter: {id: $autoRoleId}) {
          autoRoleRuleCount
          autoRoleRules(filter: $filter, page: $page, sort: $sort) {
            id
            emojiName
            roleId
            roleName

            autoRole {
              id
            }

            createdAt
            modifiedAt
          }
        }
      }
    }
  `;
}
